"""Extensions for virtualenv Seeders to pre-install artifacts-keyring and deps."""
from pathlib import Path

from pkg_resources import resource_filename
from virtualenv.seed.embed.base_embed import BaseEmbed
from virtualenv.seed.embed.pip_invoke import PipInvoke
from virtualenv.seed.wheels import bundle, embed, Version, Wheel


EXT_DIR = Path(resource_filename('azure_devops_artifacts_helpers.wheels', ''))


for py_ver, pkgs in embed.BUNDLE_SUPPORT.items():
    for pkg in EXT_DIR.glob('artifacts_keyring*.whl'):
        pkgs[pkg.name.split('-')[0]] = pkg.name


def get_embed_wheel(distribution, for_py_version):
    """Get the embed wheel from the normal virtualenv embed dir or this one."""
    wheel = (embed.BUNDLE_SUPPORT.get(for_py_version, {}) or embed.BUNDLE_SUPPORT[embed.MAX]).get(distribution)
    if wheel is not None:
        path = embed.BUNDLE_FOLDER / wheel
        if not path.exists():
            path = EXT_DIR/wheel
        wheel = Wheel.from_path(path)
    return wheel


bundle.get_embed_wheel = get_embed_wheel


class AzureDevopsArtifactsMixin(BaseEmbed):
    """Mixin to add/configure the devops artifacts dependencies."""
    def __init__(self, options):
        """Add the extra attributes for the extensions."""
        for dist in self.distributions().keys():
            setattr(self, f'no_{dist}', getattr(options, f'no_{dist}'))
            setattr(self, f'{dist}_version', getattr(options, dist))
        super(AzureDevopsArtifactsMixin, self).__init__(options)
        self.extra_search_dir.append(EXT_DIR)

    @classmethod
    def distributions(cls):
        """Return the dictionary of distributions."""
        base = super().distributions()
        for pkg in EXT_DIR.glob('artifacts_keyring*.whl'):
            base[pkg.name.split('-')[0]] = Version.bundle
        return base


class AzureDevopsArtifactsPipInvoke(AzureDevopsArtifactsMixin, PipInvoke):
    """Mixed in Azure Devops artifacts-keyring into seed packages for pip seeder."""



"""Extension for tox to use on environment creation to install artifacts-keyring to virtualenv automatically."""
import sys

try:
    import tox
    from tox.venv import cleanup_for_venv, _SKIP_VENV_CREATION, reporter

except (ImportError, ModuleNotFoundError):  # pragma: no cover
    tox = False


if tox:

    @tox.hookimpl
    def tox_testenv_create(venv, action):
        """Hook for creating tox testenv, monkeypatched from tox."""
        config_interpreter = venv.getsupportedinterpreter()
        args = [sys.executable, "-m", "virtualenv"]
        if venv.envconfig.sitepackages:
            args.append("--system-site-packages")
        if venv.envconfig.alwayscopy:
            args.append("--always-copy")
        if not venv.envconfig.download:
            args.append("--no-download")
        # add interpreter explicitly, to prevent using default (virtualenv.ini)
        args.extend(["--python", str(config_interpreter)])
        # Add seeder explicitly
        args.extend(["--seeder", 'azdo-pip'])

        cleanup_for_venv(venv)

        base_path = venv.path.dirpath()
        base_path.ensure(dir=1)
        args.append(venv.path.basename)
        if not _SKIP_VENV_CREATION:
            try:
                venv._pcall(
                    args,
                    venv=False,
                    action=action,
                    cwd=base_path,
                    redirect=reporter.verbosity() < reporter.Verbosity.DEBUG,
                )
            except KeyboardInterrupt:  # pragma: no cover
                venv.status = "keyboardinterrupt"
                raise
        # Return non-None to indicate plugin has completed
        return True


else:  # pragma: no cover
    def tox_venv(venv, action):
        """Empty method if tox not installed."""



#!/usr/bin/env python
"""
Setup script for azure_devops_artifacts_helpers
===============================================

Call from command line as::

    python setup.py --help

to see the options available.
"""
import json
import os
from pathlib import Path
import subprocess
import sys

from setuptools import setup
from setuptools.config import read_configuration
from pkg_resources import parse_version, parse_requirements

try:
    import versioneer
    __version__ = versioneer.get_version()
    cmdclass = versioneer.get_cmdclass()
except AttributeError:
    __version__ = '0.0.0'
    cmdclass = None


with open(str(Path(__file__).parent.absolute()/'supported_python_versions.json')) as f:
    PYTHON_VERSIONS = json.load(f)

# We are going to take the approach that the requirements.txt specifies
# exact (pinned versions) to use but install_requires should only
# specify package names
# see https://caremad.io/posts/2013/07/setup-vs-requirement/
# install_requires should specify abstract requirements e.g.::
#
#   install_requires = ['requests']
#
# whereas the requirements.txt file should specify pinned versions to
# generate a repeatable environment

with open('requirements.txt') as f:
    install_requires = []
    for req in parse_requirements(f.read()):
        install_requires.append(str(req).replace('==', '>='))

if parse_version(__version__) < parse_version('0.2.0'):
    development_status = 'Development Status :: 2 - Pre-Alpha'
elif parse_version(__version__).is_prerelease :
    development_status = 'Development Status :: 4 - Beta'
elif parse_version(__version__) >= parse_version('0.2.0'):
    development_status = 'Development Status :: 5 - Production/Stable'
elif parse_version(__version__) >= parse_version('1.0.0'):
    development_status = 'Development Status :: 6 - Mature'
else:
    development_status = 'Development Status :: 1 - Planning'

config = read_configuration('setup.cfg')
# See https://pypi.org/classifiers/
classifiers = config['metadata']['classifiers']
classifiers.append(development_status)
for py_version in PYTHON_VERSIONS:
    classifiers.append('Programming Language :: Python :: '+py_version)


kwargs = {'install_requires': install_requires,
          'version':  __version__,
          'classifiers': classifiers}


if cmdclass is not None:
    kwargs['cmdclass'] = cmdclass

if __name__ == "__main__":
    setup(**kwargs)